@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -61,7 +61,7 @@ Branch_delete(Branch *self, PyObject *args)
 
 
 PyDoc_STRVAR(Branch_is_head__doc__,
-  "is_head()\n"
+  "is_head() -> bool\n"
   "\n"
   "True if HEAD points at the branch, False otherwise.");
 
@@ -82,7 +82,7 @@ Branch_is_head(Branch *self)
 }
 
 PyDoc_STRVAR(Branch_is_checked_out__doc__,
-  "is_checked_out()\n"
+  "is_checked_out() -> bool\n"
   "\n"
   "True if branch is checked out by any repo connected to the current one, "
   " False otherwise.");
@@ -105,7 +105,7 @@ Branch_is_checked_out(Branch *self)
 
 
 PyDoc_STRVAR(Branch_rename__doc__,
-  "rename(name, force=False)\n"
+  "rename(name: str, force: bool = False)\n"
   "\n"
   "Move/rename an existing local branch reference. The new branch name will be "
   "checked for validity.\n"
@@ -168,7 +168,12 @@ Branch_raw_branch_name__get__(Branch *self)
 }
 
 PyDoc_STRVAR(Branch_remote_name__doc__,
-  "The name of the remote set to be the upstream of this branch.");
+    "Find the remote name of a remote-tracking branch.\n"
+    "\n"
+    "This will return the name of the remote whose fetch refspec is matching "
+    "the given branch. E.g. given a branch 'refs/remotes/test/master', it will "
+    "extract the 'test' part. If refspecs from multiple remotes match, the "
+    "function will raise ValueError.");
 
 PyObject *
 Branch_remote_name__get__(Branch *self)
@@ -185,7 +190,7 @@ Branch_remote_name__get__(Branch *self)
     if (err < GIT_OK)
         return Error_set(err);
 
-    py_name = to_unicode(name.ptr, NULL, NULL);
+    py_name = to_unicode_n(name.ptr, name.size, NULL, NULL);
     git_buf_dispose(&name);
 
     return py_name;
@@ -263,7 +268,7 @@ Branch_upstream_name__get__(Branch *self)
     if (err < GIT_OK)
         return Error_set(err);
 
-    py_name = to_unicode(name.ptr, NULL, NULL);
+    py_name = to_unicode_n(name.ptr, name.size, NULL, NULL);
     git_buf_dispose(&name);
 
     return py_name;
