@@ -179,6 +179,31 @@ DiffFile_dealloc(DiffFile *self)
     PyObject_Del(self);
 }
 
+PyDoc_STRVAR(DiffFile_from_c__doc__, "Method exposed for _checkout_notify_cb to hook into");
+
+/* Expose wrap_diff_file to python so we can call it in callbacks.py. */
+PyObject *
+DiffFile_from_c(DiffFile *dummy, PyObject *py_diff_file_ptr)
+{
+    const git_diff_file *diff_file;
+    char *buffer;
+    Py_ssize_t length;
+
+    /* Here we need to do the opposite conversion from the _pointer getters */
+    if (PyBytes_AsStringAndSize(py_diff_file_ptr, &buffer, &length))
+        return NULL;
+
+    if (length != sizeof(git_diff_file *)) {
+        PyErr_SetString(PyExc_TypeError, "passed value is not a pointer");
+        return NULL;
+    }
+
+    /* the "buffer" contains the pointer */
+    diff_file = *((const git_diff_file **) buffer);
+
+    return wrap_diff_file(diff_file);
+}
+
 PyMemberDef DiffFile_members[] = {
     MEMBER(DiffFile, id, T_OBJECT, "Oid of the item."),
     MEMBER(DiffFile, path, T_STRING, "Path to the entry."),
@@ -189,6 +214,10 @@ PyMemberDef DiffFile_members[] = {
     {NULL}
 };
 
+PyMethodDef DiffFile_methods[] = {
+    METHOD(DiffFile, from_c, METH_STATIC | METH_O),
+    {NULL},
+};
 
 PyDoc_STRVAR(DiffFile__doc__, "DiffFile object.");
 
@@ -220,7 +249,7 @@ PyTypeObject DiffFileType = {
     0,                                         /* tp_weaklistoffset */
     0,                                         /* tp_iter           */
     0,                                         /* tp_iternext       */
-    0,                                         /* tp_methods        */
+    DiffFile_methods,                          /* tp_methods        */
     DiffFile_members,                          /* tp_members        */
     0,                                         /* tp_getset         */
     0,                                         /* tp_base           */
@@ -235,7 +264,7 @@ PyTypeObject DiffFileType = {
 
 
 PyDoc_STRVAR(DiffDelta_status_char__doc__,
-  "status_char()\n"
+  "status_char() -> str\n"
   "\n"
   "Return the single character abbreviation for a delta status code."
 );
@@ -617,35 +646,16 @@ PyDoc_STRVAR(Diff_patch__doc__,
 PyObject *
 Diff_patch__get__(Diff *self)
 {
-    git_patch* patch;
     git_buf buf = {NULL};
-    int err = GIT_ERROR;
-    size_t i, num;
-    PyObject *py_patch = NULL;
 
-    num = git_diff_num_deltas(self->diff);
-    if (num == 0)
-        Py_RETURN_NONE;
+    int err = git_diff_to_buf(&buf, self->diff, GIT_DIFF_FORMAT_PATCH);
+    if (err < 0)
+        return Error_set(err);
 
-    for (i = 0; i < num ; ++i) {
-        err = git_patch_from_diff(&patch, self->diff, i);
-        if (err < 0)
-            goto cleanup;
+    PyObject *py_patch = to_unicode_n(buf.ptr, buf.size, NULL, NULL);
 
-        /* This appends to the current buf, so we can simply keep passing it */
-        err = git_patch_to_buf(&buf, patch);
-        if (err < 0)
-            goto cleanup;
-
-        git_patch_free(patch);
-    }
-
-    py_patch = to_unicode_n(buf.ptr, buf.size, NULL, NULL);
     git_buf_dispose(&buf);
-
-cleanup:
-    git_buf_dispose(&buf);
-    return (err < 0) ? Error_set(err) : py_patch;
+    return py_patch;
 }
 
 
@@ -803,7 +813,7 @@ DiffStats_files_changed__get__(DiffStats *self)
 }
 
 PyDoc_STRVAR(DiffStats_format__doc__,
-    "format(format, width)\n"
+    "format(format: int, width: int) -> str\n"
     "\n"
     "Format the stats as a string.\n"
     "\n"
@@ -935,7 +945,7 @@ Diff_from_c(Diff *dummy, PyObject *args)
 }
 
 PyDoc_STRVAR(Diff_merge__doc__,
-  "merge(diff)\n"
+  "merge(diff: Diff)\n"
   "\n"
   "Merge one diff into another.");
 
@@ -957,7 +967,7 @@ Diff_merge(Diff *self, PyObject *args)
 
 
 PyDoc_STRVAR(Diff_find_similar__doc__,
-  "find_similar([flags, rename_threshold, copy_threshold, rename_from_rewrite_threshold, break_rewrite_threshold, rename_limit])\n"
+  "find_similar(flags: int = GIT_DIFF_FIND_BY_CONFIG, rename_threshold: int = 50, copy_threshold: int = 50, rename_from_rewrite_threshold: int = 50, break_rewrite_threshold: int = 60, rename_limit: int = 1000)\n"
   "\n"
   "Transform a diff marking file renames, copies, etc.\n"
   "\n"
@@ -1028,7 +1038,7 @@ Diff_stats__get__(Diff *self)
 }
 
 PyDoc_STRVAR(Diff_parse_diff__doc__,
-    "parse_diff(git_diff: str) -> Diff\n"
+    "parse_diff(git_diff: str | bytes) -> Diff\n"
     "\n"
     "Parses a git unified diff into a diff object without a repository");
 
