@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -27,9 +27,9 @@
 
 #include "error.h"
 
-PyObject *GitError;
-PyObject *AlreadyExistsError;
-PyObject *InvalidSpecError;
+extern PyObject *GitError;
+extern PyObject *AlreadyExistsError;
+extern PyObject *InvalidSpecError;
 
 PyObject *
 Error_type(int type)
@@ -105,14 +105,13 @@ Error_set_exc(PyObject* exception)
 PyObject *
 Error_set_str(int err, const char *str)
 {
-    const git_error* error;
     if (err == GIT_ENOTFOUND) {
         /* KeyError expects the arg to be the missing key. */
         PyErr_SetString(PyExc_KeyError, str);
         return NULL;
     }
 
-    error = git_error_last();
+    const git_error *error = git_error_last();
     if (error == NULL) /* Expected error - no error msg set */
         return PyErr_Format(Error_type(err), "%s", str);
 
@@ -135,3 +134,28 @@ Error_type_error(const char *format, PyObject *value)
     PyErr_Format(PyExc_TypeError, format, Py_TYPE(value)->tp_name);
     return NULL;
 }
+
+int
+git_error_for_exc(void)
+{
+    PyObject *err = PyErr_Occurred();
+    if (err) {
+        // FIXME Here we're masking exception, if the Python implementation has
+        // a genuine Key or Value error. We should have an explicit way for the
+        // Python callbacks to signal ENOTFOUND (and EAMBIGUOUS?)
+
+        // Not found is an expected condition (the ODB will try with the next
+        // backend), so we clear the exception.
+        if (PyErr_GivenExceptionMatches(err, PyExc_KeyError)) {
+            PyErr_Clear();
+            return GIT_ENOTFOUND;
+        }
+
+        if (PyErr_GivenExceptionMatches(err, PyExc_ValueError))
+            return GIT_EAMBIGUOUS;
+
+        return GIT_EUSER;
+    }
+
+    return 0;
+}
