@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -80,7 +80,7 @@ Patch_delta__get__(Patch *self)
 }
 
 PyDoc_STRVAR(Patch_line_stats__doc__,
-    "Get line counts of each type in a patch.");
+    "Get line counts of each type in a patch (context, additions, deletions).");
 
 PyObject *
 Patch_line_stats__get__(Patch *self)
@@ -207,27 +207,11 @@ Patch_text__get__(Patch *self)
     if (err < 0)
         return Error_set(err);
 
-    text = to_unicode(buf.ptr, NULL, NULL);
+    text = to_unicode_n(buf.ptr, buf.size, NULL, NULL);
     git_buf_dispose(&buf);
     return text;
 }
 
-PyDoc_STRVAR(Patch_patch__doc__,
-    "Patch diff string (deprecated -- use Patch.text instead).\n"
-    "Can be None in some cases, such as empty commits. "
-    "Note that this decodes the content to Unicode assuming UTF-8 encoding. "
-    "For non-UTF-8 content that can lead be a lossy, non-reversible process. "
-    "To access the raw, un-decoded patch, use `patch.data`.");
-
-PyObject *
-Patch_patch__get__(Patch *self)
-{
-    PyErr_WarnEx(PyExc_DeprecationWarning,
-                 "Patch.patch is deprecated, use Patch.text instead",
-                 1);
-    return Patch_text__get__(self);
-}
-
 PyDoc_STRVAR(Patch_hunks__doc__, "hunks");
 
 PyObject *
@@ -262,7 +246,6 @@ PyGetSetDef Patch_getsetters[] = {
     GETTER(Patch, line_stats),
     GETTER(Patch, data),
     GETTER(Patch, text),
-    GETTER(Patch, patch),
     GETTER(Patch, hunks),
     {NULL}
 };
