@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -36,10 +36,12 @@
 #include "oid.h"
 #include "signature.h"
 #include "reference.h"
+#include <git2/sys/refs.h>
 
 
 extern PyObject *GitError;
 extern PyTypeObject RefLogEntryType;
+extern PyTypeObject RepositoryType;
 extern PyTypeObject SignatureType;
 
 PyTypeObject ReferenceType;
@@ -115,6 +117,62 @@ PyTypeObject RefLogIterType = {
     (iternextfunc)RefLogIter_iternext,         /* tp_iternext       */
 };
 
+PyDoc_STRVAR(Reference__doc__,
+        "Reference(name: str, target: str): create a symbolic reference\n"
+        "\n"
+        "Reference(name: str, oid: Oid, peel: Oid): create a direct reference\n"
+        "\n"
+        "'peel' is the first non-tag object's OID, or None.\n"
+        "\n"
+        "The purpose of this constructor is for use in custom refdb backends.\n"
+        "References created with this function are unlikely to work as\n"
+        "expected in other contexts.\n");
+
+static int
+Reference_init_symbolic(Reference *self, PyObject *args, PyObject *kwds)
+{
+    const char *name, *target;
+    if (!PyArg_ParseTuple(args, "ss", &name, &target)) {
+        return -1;
+    }
+    self->reference = git_reference__alloc_symbolic(name, target);
+    return 0;
+}
+
+int
+Reference_init(Reference *self, PyObject *args, PyObject *kwds)
+{
+    const char *name;
+    git_oid oid, peel;
+    PyObject *py_oid, *py_peel;
+
+    if (kwds && PyDict_Size(kwds) > 0) {
+        PyErr_SetString(PyExc_TypeError, "Reference takes no keyword arguments");
+        return -1;
+    }
+
+    Py_ssize_t nargs = PyTuple_Size(args);
+    if (nargs == 2) {
+        return Reference_init_symbolic(self, args, kwds);
+    } else if (nargs != 3) {
+        PyErr_SetString(PyExc_TypeError,
+                "Invalid arguments to Reference constructor");
+        return -1;
+    }
+
+    if (!PyArg_ParseTuple(args, "sOO", &name, &py_oid, &py_peel)) {
+        return -1;
+    }
+
+    py_oid_to_git_oid(py_oid, &oid);
+    if (py_peel != Py_None) {
+        py_oid_to_git_oid(py_peel, &peel);
+    }
+
+    self->reference = git_reference__alloc(name, &oid, &peel);
+    return 0;
+}
+
 void
 Reference_dealloc(Reference *self)
 {
@@ -149,29 +207,27 @@ Reference_delete(Reference *self, PyObject *args)
 
 
 PyDoc_STRVAR(Reference_rename__doc__,
-  "rename(new_name)\n"
+  "rename(new_name: str)\n"
   "\n"
   "Rename the reference.");
 
 PyObject *
 Reference_rename(Reference *self, PyObject *py_name)
 {
-    char *c_name;
-    int err;
-    git_reference *new_reference;
-
     CHECK_REFERENCE(self);
 
     /* Get the C name */
-    c_name = py_path_to_c_str(py_name);
+    char *c_name = pgit_encode_fsdefault(py_name);
     if (c_name == NULL)
         return NULL;
 
     /* Rename */
-    err = git_reference_rename(&new_reference, self->reference, c_name, 0, NULL);
-    git_reference_free(self->reference);
+    git_reference *new_reference;
+    int err = git_reference_rename(&new_reference, self->reference, c_name, 0, NULL);
     free(c_name);
-    if (err < 0)
+
+    git_reference_free(self->reference);
+    if (err)
         return Error_set(err);
 
     self->reference = new_reference;
@@ -207,6 +263,23 @@ Reference_resolve(Reference *self, PyObject *args)
 }
 
 
+static PyObject *
+Reference_target_impl(Reference *self, const char ** c_name)
+{
+    CHECK_REFERENCE(self);
+
+    /* Case 1: Direct */
+    if (GIT_REF_OID == git_reference_type(self->reference))
+        return git_oid_to_python(git_reference_target(self->reference));
+
+    /* Case 2: Symbolic */
+    *c_name = git_reference_symbolic_target(self->reference);
+    if (*c_name == NULL)
+        PyErr_SetString(PyExc_ValueError, "no target available");
+
+    return NULL;
+}
+
 PyDoc_STRVAR(Reference_target__doc__,
     "The reference target: If direct the value will be an Oid object, if it\n"
     "is symbolic it will be an string with the full name of the target\n"
@@ -215,25 +288,39 @@ PyDoc_STRVAR(Reference_target__doc__,
 PyObject *
 Reference_target__get__(Reference *self)
 {
-    const char * c_name;
+    const char * c_name = NULL;
+    PyObject * ret;
 
-    CHECK_REFERENCE(self);
+    ret = Reference_target_impl(self, &c_name);
+    if (ret != NULL)
+        return ret;
+    if (c_name != NULL)
+        return to_path(c_name);
+    return NULL;
+}
 
-    /* Case 1: Direct */
-    if (GIT_REF_OID == git_reference_type(self->reference))
-        return git_oid_to_python(git_reference_target(self->reference));
 
-    /* Case 2: Symbolic */
-    c_name = git_reference_symbolic_target(self->reference);
-    if (c_name == NULL) {
-        PyErr_SetString(PyExc_ValueError, "no target available");
-        return NULL;
-    }
-    return to_path(c_name);
+PyDoc_STRVAR(Reference_raw_target__doc__,
+    "The raw reference target: If direct the value will be an Oid object, if it\n"
+    "is symbolic it will be bytes with the full name of the target\n"
+    "reference.\n");
+
+PyObject *
+Reference_raw_target__get__(Reference *self)
+{
+    const char * c_name = NULL;
+    PyObject * ret;
+
+    ret = Reference_target_impl(self, &c_name);
+    if (ret != NULL)
+        return ret;
+    if (c_name != NULL)
+        return PyBytes_FromString(c_name);
+    return NULL;
 }
 
 PyDoc_STRVAR(Reference_set_target__doc__,
-    "set_target(target, [message])\n"
+    "set_target(target[, message: str])\n"
     "\n"
     "Set the target of this reference.\n"
     "\n"
@@ -253,7 +340,6 @@ PyObject *
 Reference_set_target(Reference *self, PyObject *args, PyObject *kwds)
 {
     git_oid oid;
-    char *c_name;
     int err;
     git_reference *new_ref;
     PyObject *py_target = NULL;
@@ -282,7 +368,7 @@ Reference_set_target(Reference *self, PyObject *args, PyObject *kwds)
     }
 
     /* Case 2: Symbolic */
-    c_name = py_path_to_c_str(py_target);
+    char *c_name = pgit_encode_fsdefault(py_target);
     if (c_name == NULL)
         return NULL;
 
@@ -382,7 +468,7 @@ Reference_log(Reference *self)
 }
 
 PyDoc_STRVAR(Reference_peel__doc__,
-  "peel(type=None) -> object\n"
+  "peel(type=None) -> Object\n"
   "\n"
   "Retrieve an object of the given type by recursive peeling.\n"
   "\n"
@@ -579,13 +665,11 @@ PyGetSetDef Reference_getseters[] = {
     GETTER(Reference, shorthand),
     GETTER(Reference, raw_shorthand),
     GETTER(Reference, target),
+    GETTER(Reference, raw_target),
     GETTER(Reference, type),
     {NULL}
 };
 
-
-PyDoc_STRVAR(Reference__doc__, "Reference.");
-
 PyTypeObject ReferenceType = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "_pygit2.Reference",                       /* tp_name           */
@@ -622,7 +706,7 @@ PyTypeObject ReferenceType = {
     0,                                         /* tp_descr_get      */
     0,                                         /* tp_descr_set      */
     0,                                         /* tp_dictoffset     */
-    0,                                         /* tp_init           */
+    (initproc)Reference_init,                  /* tp_init           */
     0,                                         /* tp_alloc          */
     0,                                         /* tp_new            */
 };
@@ -636,8 +720,8 @@ wrap_reference(git_reference * c_reference, Repository *repo)
     py_reference = PyObject_New(Reference, &ReferenceType);
     if (py_reference) {
         py_reference->reference = c_reference;
+        py_reference->repo = repo;
         if (repo) {
-            py_reference->repo = repo;
             Py_INCREF(repo);
         }
     }
