@@ -25,6 +25,7 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#include <git2/status.h>
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include "error.h"
@@ -279,7 +280,7 @@ Repository_is_shallow__get__(Repository *self)
 
 
 PyDoc_STRVAR(Repository_git_object_lookup_prefix__doc__,
-  "git_object_lookup_prefix(oid) -> Object\n"
+  "git_object_lookup_prefix(oid: Oid) -> Object\n"
   "\n"
   "Returns the Git object with the given oid.");
 
@@ -307,7 +308,7 @@ Repository_git_object_lookup_prefix(Repository *self, PyObject *key)
 
 
 PyDoc_STRVAR(Repository_lookup_branch__doc__,
-  "lookup_branch(branch_name, [branch_type]) -> Branch\n"
+  "lookup_branch(branch_name: str, branch_type: int = GIT_BRANCH_LOCAL) -> Branch\n"
   "\n"
   "Returns the Git reference for the given branch name (local or remote).\n"
   "If branch_type is GIT_BRANCH_REMOTE, you must include the remote name\n"
@@ -337,6 +338,8 @@ Repository_lookup_branch(Repository *self, PyObject *args)
 
 
 PyDoc_STRVAR(Repository_path_is_ignored__doc__,
+  "path_is_ignored(path: str) -> bool\n"
+  "\n"
   "Check if a path is ignored in the repository.");
 
 PyObject *
@@ -357,7 +360,7 @@ Repository_path_is_ignored(Repository *self, PyObject *args)
 
 
 PyDoc_STRVAR(Repository_revparse_single__doc__,
-  "revparse_single(revision) -> Object\n"
+  "revparse_single(revision: str) -> Object\n"
   "\n"
   "Find an object, as specified by a revision string. See\n"
   "`man gitrevisions`, or the documentation for `git rev-parse` for\n"
@@ -382,7 +385,7 @@ Repository_revparse_single(Repository *self, PyObject *py_spec)
 
 
 PyDoc_STRVAR(Repository_revparse_ext__doc__,
-  "revparse_ext(revision) -> (Object, Reference)\n"
+  "revparse_ext(revision: str) -> tuple[Object, Reference]\n"
   "\n"
   "Find a single object and intermediate reference, as specified by a revision\n"
   "string. See `man gitrevisions`, or the documentation for `git rev-parse`\n"
@@ -420,7 +423,7 @@ Repository_revparse_ext(Repository *self, PyObject *py_spec)
 
 
 PyDoc_STRVAR(Repository_revparse__doc__,
-  "revparse(revspec) -> RevSpec\n"
+  "revparse(revspec: str) -> RevSpec\n"
   "\n"
   "Parse a revision string for from, to, and intent. See `man gitrevisions`,\n"
   "or the documentation for `git rev-parse` for information on the syntax\n"
@@ -495,7 +498,7 @@ Repository_workdir__set__(Repository *self, PyObject *py_workdir)
 }
 
 PyDoc_STRVAR(Repository_descendant_of__doc__,
-  "descendant_of(oid, oid) -> bool\n"
+  "descendant_of(oid1: Oid, oid2: Oid) -> bool\n"
   "\n"
   "Determine if the first commit is a descendant of the second commit.\n"
   "Note that a commit is not considered a descendant of itself.");
@@ -529,7 +532,7 @@ Repository_descendant_of(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_merge_base__doc__,
-  "merge_base(oid, oid) -> Oid\n"
+  "merge_base(oid1: Oid, oid2: Oid) -> Oid\n"
   "\n"
   "Find as good common ancestors as possible for a merge.\n"
   "Returns None if there is no merge base between the commits");
@@ -619,7 +622,7 @@ out:
 
 
 PyDoc_STRVAR(Repository_merge_base_many__doc__,
-  "merge_base_many(oids) -> Oid\n"
+  "merge_base_many(oids: list[Oid]) -> Oid\n"
   "\n"
   "Find as good common ancestors as possible for an n-way merge.\n"
   "Returns None if there is no merge base between the commits");
@@ -631,7 +634,7 @@ Repository_merge_base_many(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_merge_base_octopus__doc__,
-  "merge_base_octopus(oids) -> Oid\n"
+  "merge_base_octopus(oids: list[Oid]) -> Oid\n"
   "\n"
   "Find as good common ancestors as possible for an n-way octopus merge.\n"
   "Returns None if there is no merge base between the commits");
@@ -643,7 +646,7 @@ Repository_merge_base_octopus(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_merge_analysis__doc__,
-  "merge_analysis(their_head, our_ref='HEAD') -> (Integer, Integer)\n"
+  "merge_analysis(their_head: Oid, our_ref: str = \"HEAD\") -> tuple[int, int]\n"
   "\n"
   "Analyzes the given branch and determines the opportunities for\n"
   "merging it into a reference (defaults to HEAD).\n"
@@ -709,7 +712,7 @@ out:
 }
 
 PyDoc_STRVAR(Repository_merge__doc__,
-  "merge(id)\n"
+  "merge(id: Oid)\n"
   "\n"
   "Merges the given id into HEAD.\n"
   "\n"
@@ -750,7 +753,7 @@ Repository_merge(Repository *self, PyObject *py_oid)
 }
 
 PyDoc_STRVAR(Repository_cherrypick__doc__,
-  "cherrypick(id)\n"
+  "cherrypick(id: Oid)\n"
   "\n"
   "Cherry-pick the given oid, producing changes in the index and working directory.\n"
   "\n"
@@ -790,9 +793,9 @@ Repository_cherrypick(Repository *self, PyObject *py_oid)
 }
 
 PyDoc_STRVAR(Repository_walk__doc__,
-    "walk(oid[, sort_mode]) -> iterator\n"
+    "walk(oid: Oid | None, sort_mode: int = GIT_SORT_NONE) -> Walker\n"
     "\n"
-    "Generator that traverses the history starting from the given commit.\n"
+    "Start traversing the history from the given commit.\n"
     "The following types of sorting could be used to control traversing\n"
     "direction:\n"
     "\n"
@@ -866,7 +869,7 @@ error:
 
 
 PyDoc_STRVAR(Repository_create_blob__doc__,
-    "create_blob(data) -> Oid\n"
+    "create_blob(data: bytes) -> Oid\n"
     "\n"
     "Create a new blob from a bytes string. The blob is added to the Git\n"
     "object database. Returns the oid of the blob.");
@@ -891,7 +894,7 @@ Repository_create_blob(Repository *self, PyObject *args)
 
 
 PyDoc_STRVAR(Repository_create_blob_fromworkdir__doc__,
-    "create_blob_fromworkdir(path) -> Oid\n"
+    "create_blob_fromworkdir(path: str) -> Oid\n"
     "\n"
     "Create a new blob from a file within the working directory. The given\n"
     "path must be relative to the working directory, if it is not an error\n"
@@ -917,7 +920,7 @@ Repository_create_blob_fromworkdir(Repository *self, PyObject *py_path)
 
 
 PyDoc_STRVAR(Repository_create_blob_fromdisk__doc__,
-    "create_blob_fromdisk(path) -> Oid\n"
+    "create_blob_fromdisk(path: str) -> Oid\n"
     "\n"
     "Create a new blob from a file anywhere (no working directory check).");
 
@@ -1025,7 +1028,7 @@ cleanup:
 
 
 PyDoc_STRVAR(Repository_create_commit__doc__,
-  "create_commit(reference_name, author, committer, message, tree, parents[, encoding]) -> Oid\n"
+  "create_commit(reference_name: str, author: Signature, committer: Signature, message: bytes | str, tree: Oid, parents: list[Oid][, encoding: str]) -> Oid\n"
   "\n"
   "Create a new commit object, return its oid.");
 
@@ -1108,9 +1111,118 @@ out:
     return py_result;
 }
 
+PyDoc_STRVAR(Repository_create_commit_string__doc__,
+  "create_commit_string(author: Signature, committer: Signature, message: bytes | str, tree: Oid, parents: list[Oid][, encoding: str]) -> str\n"
+  "\n"
+  "Create a new commit but return it as a string.");
+
+PyObject *
+Repository_create_commit_string(Repository *self, PyObject *args)
+{
+    Signature *py_author, *py_committer;
+    PyObject *py_oid, *py_message, *py_parents, *py_parent;
+    PyObject *str;
+    char *encoding = NULL;
+    git_oid oid;
+    git_tree *tree = NULL;
+    int parent_count;
+    git_commit **parents = NULL;
+    git_buf buf = { 0 };
+    int i = 0;
+
+    if (!PyArg_ParseTuple(args, "O!O!OOO!|s",
+                          &SignatureType, &py_author,
+                          &SignatureType, &py_committer,
+                          &py_message,
+                          &py_oid,
+                          &PyList_Type, &py_parents,
+                          &encoding))
+        return NULL;
+
+    size_t len = py_oid_to_git_oid(py_oid, &oid);
+    if (len == 0)
+        return NULL;
+
+    PyObject *tmessage;
+    const char *message = pgit_borrow_encoding(py_message, encoding, NULL, &tmessage);
+    if (message == NULL)
+        return NULL;
+
+    int err = git_tree_lookup_prefix(&tree, self->repo, &oid, len);
+    if (err < 0) {
+        Error_set(err);
+        goto out;
+    }
+
+    parent_count = (int)PyList_Size(py_parents);
+    parents = malloc(parent_count * sizeof(git_commit*));
+    if (parents == NULL) {
+        PyErr_SetNone(PyExc_MemoryError);
+        goto out;
+    }
+    for (; i < parent_count; i++) {
+        py_parent = PyList_GET_ITEM(py_parents, i);
+        len = py_oid_to_git_oid(py_parent, &oid);
+        if (len == 0)
+            goto out;
+        err = git_commit_lookup_prefix(&parents[i], self->repo, &oid, len);
+        if (err < 0) {
+            Error_set(err);
+            goto out;
+        }
+    }
+
+    err = git_commit_create_buffer(&buf, self->repo,
+                                   py_author->signature, py_committer->signature,
+                                   encoding, message, tree, parent_count,
+                                   (const git_commit**)parents);
+    if (err < 0) {
+        Error_set(err);
+        goto out;
+    }
+
+    str = to_unicode_n(buf.ptr, buf.size, NULL, NULL);
+    git_buf_dispose(&buf);
+
+out:
+    Py_DECREF(tmessage);
+    git_tree_free(tree);
+    while (i > 0) {
+        i--;
+        git_commit_free(parents[i]);
+    }
+    free(parents);
+    return str;
+}
+
+PyDoc_STRVAR(Repository_create_commit_with_signature__doc__,
+  "create_commit_with_signature(content: str, signature: str, field_name: str) -> Oid\n"
+  "\n"
+  "Create a new signed commit object, return its oid.");
+
+PyObject *
+Repository_create_commit_with_signature(Repository *self, PyObject *args)
+{
+    git_oid oid;
+    char *content, *signature;
+    char *signature_field = NULL;
+
+    if (!PyArg_ParseTuple(args, "ss|s", &content, &signature, &signature_field))
+        return NULL;
+
+    int err = git_commit_create_with_signature(&oid, self->repo, content,
+                                               signature, signature_field);
+
+    if (err < 0) {
+        Error_set(err);
+        return NULL;
+    }
+
+    return git_oid_to_python(&oid);
+}
 
 PyDoc_STRVAR(Repository_create_tag__doc__,
-  "create_tag(name, oid, type, tagger, message) -> Oid\n"
+  "create_tag(name: str, oid: Oid, type: int, tagger: Signature[, message: str]) -> Oid\n"
   "\n"
   "Create a new tag object, return its oid.");
 
@@ -1149,7 +1261,7 @@ Repository_create_tag(Repository *self, PyObject *args)
 
 
 PyDoc_STRVAR(Repository_create_branch__doc__,
-  "create_branch(name, commit, force=False)\n"
+  "create_branch(name: str, commit: Commit, force: bool = False) -> Branch\n"
   "\n"
   "Create a new branch \"name\" which points to a commit.\n"
   "\n"
@@ -1223,7 +1335,7 @@ out:
 }
 
 PyDoc_STRVAR(Repository_listall_references__doc__,
-  "listall_references() -> [str, ...]\n"
+  "listall_references() -> list[str]\n"
   "\n"
   "Return a list with all the references in the repository.");
 
@@ -1234,7 +1346,7 @@ Repository_listall_references(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_raw_listall_references__doc__,
-  "raw_listall_references() -> [bytes, ...]\n"
+  "raw_listall_references() -> list[bytes]\n"
   "\n"
   "Return a list with all the references in the repository.");
 
@@ -1246,7 +1358,7 @@ Repository_raw_listall_references(Repository *self, PyObject *args)
 
 
 PyDoc_STRVAR(Repository_listall_reference_objects__doc__,
-  "listall_reference_objects() -> [Reference, ...]\n"
+  "listall_reference_objects() -> list[Reference]\n"
   "\n"
   "Return a list with all the reference objects in the repository.");
 
@@ -1348,7 +1460,7 @@ error:
 }
 
 PyDoc_STRVAR(Repository_listall_branches__doc__,
-  "listall_branches([flag]) -> [str, ...]\n"
+  "listall_branches(flag: int = GIT_BRANCH_LOCAL) -> list[str]\n"
   "\n"
   "Return a list with all the branches in the repository.\n"
   "\n"
@@ -1365,7 +1477,7 @@ Repository_listall_branches(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_raw_listall_branches__doc__,
-  "raw_listall_branches([flag]) -> [bytes, ...]\n"
+  "raw_listall_branches(flag: int = GIT_BRANCH_LOCAL) -> list[bytes]\n"
   "\n"
   "Return a list with all the branches in the repository.\n"
   "\n"
@@ -1382,7 +1494,7 @@ Repository_raw_listall_branches(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_listall_submodules__doc__,
-  "listall_submodules() -> [str, ...]\n"
+  "listall_submodules() -> list[str]\n"
   "\n"
   "Return a list with all submodule paths in the repository.\n");
 
@@ -1417,7 +1529,7 @@ Repository_listall_submodules(Repository *self, PyObject *args)
 
 
 PyDoc_STRVAR(Repository_init_submodules__doc__,
-    "init_submodule(submodules=None, overwrite=False)\n"
+    "init_submodules(submodules: list[Submodule] = None, overwrite=False)\n"
     "\n"
     "Initialize all submodules in repository.\n"
     "submodules: List of submodules to initialize. Default argument initializes all submodules.\n"
@@ -1492,7 +1604,7 @@ error:
 }
 
 PyDoc_STRVAR(Repository_lookup_reference__doc__,
-  "lookup_reference(name) -> Reference\n"
+  "lookup_reference(name: str) -> Reference\n"
   "\n"
   "Lookup a reference by its name in a repository.");
 
@@ -1519,7 +1631,7 @@ Repository_lookup_reference(Repository *self, PyObject *py_name)
 }
 
 PyDoc_STRVAR(Repository_lookup_reference_dwim__doc__,
-  "lookup_reference_dwim(name) -> Reference\n"
+  "lookup_reference_dwim(name: str) -> Reference\n"
   "\n"
   "Lookup a reference by doing-what-i-mean'ing its short name.");
 
@@ -1546,7 +1658,7 @@ Repository_lookup_reference_dwim(Repository *self, PyObject *py_name)
 }
 
 PyDoc_STRVAR(Repository_create_reference_direct__doc__,
-  "create_reference_direct(name, target, force, message=None)\n"
+  "create_reference_direct(name: str, target: Oid, force: bool, message=None) -> Reference\n"
   "\n"
   "Create a new reference \"name\" which points to an object.\n"
   "\n"
@@ -1593,7 +1705,7 @@ Repository_create_reference_direct(Repository *self,  PyObject *args,
 }
 
 PyDoc_STRVAR(Repository_create_reference_symbolic__doc__,
-  "create_reference_symbolic(name, source, force, message=None)\n"
+  "create_reference_symbolic(name: str, target: str, force: bool, message: str = None) -> Reference\n"
   "\n"
   "Create a new reference \"name\" which points to another reference.\n"
   "\n"
@@ -1660,20 +1772,65 @@ Repository_compress_references(Repository *self)
 }
 
 PyDoc_STRVAR(Repository_status__doc__,
-  "status() -> {str: int}\n"
+  "status(untracked_files: str = \"all\", ignored: bool = False) -> dict[str, int]\n"
   "\n"
   "Reads the status of the repository and returns a dictionary with file\n"
-  "paths as keys and status flags as values. See pygit2.GIT_STATUS_*.");
+  "paths as keys and status flags as values. See pygit2.GIT_STATUS_*.\n"
+  "\n"
+  "Parameters:\n"
+  "\n"
+  "untracked_files\n"
+  "    How to handle untracked files, defaults to \"all\":\n"
+  "\n"
+  "    - \"no\": do not return untracked files\n"
+  "    - \"normal\": include untracked files/directories but no dot recurse subdirectories\n"
+  "    - \"all\": include all files in untracked directories\n"
+  "\n"
+  "    Using `untracked_files=\"no\"` or \"normal\"can be faster than \"all\" when the worktree\n"
+  "    contains many untracked files/directories.\n"
+  "\n"
+  "ignored\n"
+  "    Whether to show ignored files with untracked files. Ignored when untracked_files == \"no\"\n"
+  "    Defaults to False.\n");
 
 PyObject *
-Repository_status(Repository *self)
+Repository_status(Repository *self, PyObject *args, PyObject *kw)
 {
     PyObject *dict;
     int err;
     size_t len, i;
     git_status_list *list;
 
-    err = git_status_list_new(&list, self->repo, NULL);
+    char *untracked_files = "all";
+    static char *kwlist[] = {"untracked_files", "ignored", NULL};
+
+    PyObject* ignored = Py_False;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kw, "|sO", kwlist,
+                                     &untracked_files, &ignored))
+        goto error;
+
+    git_status_options opts = GIT_STATUS_OPTIONS_INIT;
+    opts.flags = GIT_STATUS_OPT_DEFAULTS;
+
+    if (!strcmp(untracked_files, "no")) {
+       opts.flags &= ~(GIT_STATUS_OPT_INCLUDE_UNTRACKED | GIT_STATUS_OPT_RECURSE_UNTRACKED_DIRS);
+    } else if (!strcmp(untracked_files, "normal")){
+       opts.flags &= ~GIT_STATUS_OPT_RECURSE_UNTRACKED_DIRS;
+    } else if (strcmp(untracked_files, "all") ){
+        return PyErr_Format(
+          PyExc_ValueError,
+          "untracked_files must be one of \"all\", \"normal\" or \"one\"");
+    };
+
+    if (!PyBool_Check(ignored)) {
+        return PyErr_Format(PyExc_TypeError, "ignored must be True or False");
+    }
+    if (!PyObject_IsTrue(ignored)) {
+        opts.flags &= ~GIT_STATUS_OPT_INCLUDE_IGNORED;
+    }
+
+    err = git_status_list_new(&list, self->repo, &opts);
     if (err < 0)
         return Error_set(err);
 
@@ -1717,7 +1874,7 @@ error:
 
 
 PyDoc_STRVAR(Repository_status_file__doc__,
-  "status_file(path) -> int\n"
+  "status_file(path: str) -> int\n"
   "\n"
   "Returns the status of the given file path.");
 
@@ -1879,7 +2036,7 @@ Repository_notes(Repository *self, PyObject *args)
 
 
 PyDoc_STRVAR(Repository_create_note__doc__,
-  "create_note(message, author, committer, annotated_id [,ref, force]) -> Oid\n"
+  "create_note(message: str, author: Signature, committer: Signature, annotated_id: str, ref: str = \"refs/notes/commits\", force: bool = False) -> Oid\n"
   "\n"
   "Create a new note for an object, return its SHA-ID."
   "If no ref is given 'refs/notes/commits' will be used.");
@@ -1915,7 +2072,7 @@ Repository_create_note(Repository *self, PyObject* args)
 
 
 PyDoc_STRVAR(Repository_lookup_note__doc__,
-  "lookup_note(annotated_id [, ref]) -> Note\n"
+  "lookup_note(annotated_id: str, ref: str = \"refs/notes/commits\") -> Note\n"
   "\n"
   "Lookup a note for an annotated object in a repository.");
 
@@ -1937,7 +2094,7 @@ Repository_lookup_note(Repository *self, PyObject* args)
 }
 
 PyDoc_STRVAR(Repository_reset__doc__,
-    "reset(oid, reset_type)\n"
+    "reset(oid: Oid, reset_type: int)\n"
     "\n"
     "Resets the current head.\n"
     "\n"
@@ -1997,7 +2154,7 @@ Repository_free(Repository *self)
 }
 
 PyDoc_STRVAR(Repository_expand_id__doc__,
-    "expand_id(hex) -> Oid\n"
+    "expand_id(hex: str) -> Oid\n"
     "\n"
     "Expand a string into a full Oid according to the objects in this repsitory.\n");
 
@@ -2015,7 +2172,7 @@ Repository_expand_id(Repository *self, PyObject *py_hex)
 }
 
 PyDoc_STRVAR(Repository_add_worktree__doc__,
-    "add_worktree(name, path, [ref])\n"
+    "add_worktree(name: str, path: str | bytes[, ref: Reference]) -> Worktree\n"
     "\n"
     "Create a new worktree for this repository. If ref is specified, no new \
     branch will be created and the provided ref will be checked out instead.");
@@ -2049,7 +2206,7 @@ Repository_add_worktree(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_lookup_worktree__doc__,
-    "lookup_worktree(name) -> Worktree\n"
+    "lookup_worktree(name: str) -> Worktree\n"
     "\n"
     "Lookup a worktree from its name.");
 PyObject *
@@ -2070,7 +2227,7 @@ Repository_lookup_worktree(Repository *self, PyObject *args)
 }
 
 PyDoc_STRVAR(Repository_list_worktrees__doc__,
-    "list_worktrees() -> [str, ...]\n"
+    "list_worktrees() -> list[str]\n"
     "\n"
     "Return a list with all the worktrees of this repository.");
 PyObject *
@@ -2107,7 +2264,7 @@ out:
 }
 
 PyDoc_STRVAR(Repository_apply__doc__,
-  "apply(diff, location=GIT_APPLY_LOCATION_WORKDIR)\n"
+  "apply(diff: Diff, location: int = GIT_APPLY_LOCATION_WORKDIR)\n"
   "\n"
   "Applies the given Diff object to HEAD, writing the results into the\n"
   "working directory, the index, or both.\n"
@@ -2144,7 +2301,7 @@ Repository_apply(Repository *self, PyObject *args, PyObject *kwds)
 }
 
 PyDoc_STRVAR(Repository_applies__doc__,
-  "applies(diff, location=GIT_APPLY_LOCATION_INDEX) -> bool\n"
+  "applies(diff: Diff, location: int = GIT_APPLY_LOCATION_INDEX, raise_error: bool = False) -> bool\n"
   "\n"
   "Tests if the given patch will apply to HEAD, without writing it.\n"
   "\n"
@@ -2156,6 +2313,10 @@ PyDoc_STRVAR(Repository_applies__doc__,
   "location\n"
   "    The location to apply: GIT_APPLY_LOCATION_WORKDIR,\n"
   "    GIT_APPLY_LOCATION_INDEX (default), or GIT_APPLY_LOCATION_BOTH.\n"
+  "\n"
+  "raise_error\n"
+  "    If the patch doesn't apply, raise an exception containing more details\n"
+  "    about the failure instead of returning False.\n"
   );
 
 PyObject *
@@ -2163,19 +2324,24 @@ Repository_applies(Repository *self, PyObject *args, PyObject *kwds)
 {
     Diff *py_diff;
     int location = GIT_APPLY_LOCATION_INDEX;
+    int raise_error = 0;
     git_apply_options options = GIT_APPLY_OPTIONS_INIT;
     options.flags |= GIT_APPLY_CHECK;
 
-    char* keywords[] = {"diff", "location", NULL};
+    char* keywords[] = {"diff", "location", "raise_error", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|i", keywords,
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|ii", keywords,
                                      &DiffType, &py_diff,
-                                     &location))
+                                     &location, &raise_error))
         return NULL;
 
     int err = git_apply(self->repo, ((Diff*)py_diff)->diff, location, &options);
-    if (err != 0)
-        Py_RETURN_FALSE;
+    if (err != 0) {
+        if (raise_error)
+            return Error_set(err);
+        else
+            Py_RETURN_FALSE;
+    }
 
     Py_RETURN_TRUE;
 }
@@ -2244,7 +2410,7 @@ static int foreach_stash_cb(size_t index, const char *message, const git_oid *st
 }
 
 PyDoc_STRVAR(Repository_listall_stashes__doc__,
-  "listall_stashes() -> [Stash, ...]\n"
+  "listall_stashes() -> list[Stash]\n"
   "\n"
   "Return a list with all stashed commits in the repository.\n");
 
@@ -2275,6 +2441,8 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, create_blob_fromdisk, METH_VARARGS),
     METHOD(Repository, create_blob_fromiobase, METH_O),
     METHOD(Repository, create_commit, METH_VARARGS),
+    METHOD(Repository, create_commit_string, METH_VARARGS),
+    METHOD(Repository, create_commit_with_signature, METH_VARARGS),
     METHOD(Repository, create_tag, METH_VARARGS),
     METHOD(Repository, TreeBuilder, METH_VARARGS),
     METHOD(Repository, walk, METH_VARARGS),
@@ -2300,7 +2468,7 @@ PyMethodDef Repository_methods[] = {
     METHOD(Repository, revparse_single, METH_O),
     METHOD(Repository, revparse_ext, METH_O),
     METHOD(Repository, revparse, METH_O),
-    METHOD(Repository, status, METH_NOARGS),
+    METHOD(Repository, status, METH_VARARGS | METH_KEYWORDS),
     METHOD(Repository, status_file, METH_O),
     METHOD(Repository, notes, METH_VARARGS),
     METHOD(Repository, create_note, METH_VARARGS),
