@@ -43,6 +43,8 @@ PyObject* Repository_walk(Repository *self, PyObject *args);
 PyObject* Repository_create_blob(Repository *self, PyObject *args);
 PyObject* Repository_create_blob_fromdisk(Repository *self, PyObject *args);
 PyObject* Repository_create_commit(Repository *self, PyObject *args);
+PyObject* Repository_create_commit_string(Repository *self, PyObject *args);
+PyObject* Repository_create_commit_with_signature(Repository *self, PyObject *args);
 PyObject* Repository_create_tag(Repository *self, PyObject *args);
 PyObject* Repository_create_branch(Repository *self, PyObject *args);
 PyObject* Repository_listall_references(Repository *self, PyObject *args);
@@ -58,7 +60,7 @@ PyObject* Repository_create_reference_direct(Repository *self, PyObject *args, P
 PyObject* Repository_create_reference_symbolic(Repository *self, PyObject *args, PyObject* kw);
 
 PyObject* Repository_compress_references(Repository *self);
-PyObject* Repository_status(Repository *self);
+PyObject* Repository_status(Repository *self, PyObject *args, PyObject *kw);
 PyObject* Repository_status_file(Repository *self, PyObject *value);
 PyObject* Repository_TreeBuilder(Repository *self, PyObject *args);
 
