@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -39,30 +39,26 @@ int
 Signature_init(Signature *self, PyObject *args, PyObject *kwds)
 {
     char *keywords[] = {"name", "email", "time", "offset", "encoding", NULL};
-    PyObject *py_name, *tname;
-    char *email, *encoding = "utf-8";
-    const char *name;
+    PyObject *py_name;
+    char *email, *encoding = NULL;
     long long time = -1;
     int offset = 0;
-    int err;
-    git_signature *signature;
 
     if (!PyArg_ParseTupleAndKeywords(
-            args, kwds, "Os|Lis", keywords,
+            args, kwds, "Os|Liz", keywords,
             &py_name, &email, &time, &offset, &encoding))
         return -1;
 
-    name = py_str_borrow_c_str(&tname, py_name, encoding);
+    PyObject *tname;
+    const char *name = pgit_borrow_encoding(
+        py_name, value_or_default(encoding, "utf-8"), NULL, &tname);
     if (name == NULL)
         return -1;
 
-    if (time == -1) {
-        err = git_signature_now(&signature, name, email);
-    } else {
-        err = git_signature_new(&signature, name, email, time, offset);
-    }
+    git_signature *signature;
+    int err = (time == -1) ? git_signature_now(&signature, name, email)
+                           : git_signature_new(&signature, name, email, time, offset);
     Py_DECREF(tname);
-
     if (err < 0) {
         Error_set(err);
         return -1;
@@ -85,14 +81,16 @@ Signature_init(Signature *self, PyObject *args, PyObject *kwds)
 void
 Signature_dealloc(Signature *self)
 {
-    /* self->obj is the owner of the git_signature, so we musn't free it */
+    /* self->obj is the owner of the git_signature C structure, so we musn't free it */
     if (self->obj) {
         Py_CLEAR(self->obj);
     } else {
         git_signature_free((git_signature *) self->signature);
-        free(self->encoding);
     }
 
+    /* we own self->encoding */
+    free(self->encoding);
+
     PyObject_Del(self);
 }
 
@@ -109,11 +107,10 @@ PyDoc_STRVAR(Signature__encoding__doc__, "Encoding.");
 PyObject *
 Signature__encoding__get__(Signature *self)
 {
-    const char *encoding;
-
-    encoding = self->encoding;
-    if (encoding == NULL)
+    const char *encoding = self->encoding;
+    if (encoding == NULL) {
         encoding = "utf-8";
+    }
 
     return to_encoding(encoding);
 }
@@ -124,7 +121,7 @@ PyDoc_STRVAR(Signature_raw_name__doc__, "Name (bytes).");
 PyObject *
 Signature_raw_name__get__(Signature *self)
 {
-    return to_bytes(self->signature->name);
+    return PyBytes_FromString(self->signature->name);
 }
 
 
@@ -133,7 +130,7 @@ PyDoc_STRVAR(Signature_raw_email__doc__, "Email (bytes).");
 PyObject *
 Signature_raw_email__get__(Signature *self)
 {
-    return to_bytes(self->signature->email);
+    return PyBytes_FromString(self->signature->email);
 }
 
 
@@ -204,7 +201,9 @@ Signature_richcompare(PyObject *a, PyObject *b, int op)
         strcmp(sa->signature->email, sb->signature->email) == 0 &&
         sa->signature->when.time == sb->signature->when.time &&
         sa->signature->when.offset == sb->signature->when.offset &&
-        sa->signature->when.sign == sb->signature->when.sign);
+        sa->signature->when.sign == sb->signature->when.sign &&
+        strcmp(value_or_default(sa->encoding, "utf-8"),
+               value_or_default(sb->encoding, "utf-8")) == 0);
 
     switch (op) {
         case Py_EQ:
@@ -226,6 +225,45 @@ Signature_richcompare(PyObject *a, PyObject *b, int op)
 
 }
 
+static PyObject *
+Signature__str__(Signature *self)
+{
+    PyObject *name, *email, *str;
+    name = to_unicode(self->signature->name, self->encoding, NULL);
+    email = to_unicode(self->signature->email, self->encoding, NULL);
+    str = PyUnicode_FromFormat("%U <%U>", name, email);
+    Py_DECREF(name);
+    Py_DECREF(email);
+    return str;
+}
+
+static PyObject *
+Signature__repr__(Signature *self)
+{
+    PyObject *name, *email, *encoding, *str;
+    name = to_unicode(self->signature->name, self->encoding, NULL);
+    email = to_unicode(self->signature->email, self->encoding, NULL);
+
+    if (self->encoding) {
+        encoding = to_unicode(self->encoding, self->encoding, NULL);
+    } else {
+        encoding = Py_None;
+        Py_INCREF(Py_None);
+    }
+
+    str = PyUnicode_FromFormat(
+        "pygit2.Signature(%R, %R, %lld, %ld, %R)",
+        name,
+        email,
+        self->signature->when.time,
+        self->signature->when.offset,
+        encoding);
+    Py_DECREF(name);
+    Py_DECREF(email);
+    Py_DECREF(encoding);
+    return str;
+}
+
 
 PyDoc_STRVAR(Signature__doc__, "Signature.");
 
@@ -239,13 +277,13 @@ PyTypeObject SignatureType = {
     0,                                         /* tp_getattr        */
     0,                                         /* tp_setattr        */
     0,                                         /* tp_compare        */
-    0,                                         /* tp_repr           */
+    (reprfunc)Signature__repr__,               /* tp_repr           */
     0,                                         /* tp_as_number      */
     0,                                         /* tp_as_sequence    */
     0,                                         /* tp_as_mapping     */
     0,                                         /* tp_hash           */
     0,                                         /* tp_call           */
-    0,                                         /* tp_str            */
+    (reprfunc)Signature__str__,                /* tp_str            */
     0,                                         /* tp_getattro       */
     0,                                         /* tp_setattro       */
     0,                                         /* tp_as_buffer      */
