@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -33,14 +33,6 @@
 #include <git2.h>
 #include "types.h"
 
-PyObject* Signature_get_encoding(Signature *self);
-PyObject* Signature_get_raw_name(Signature *self);
-PyObject* Signature_get_raw_email(Signature *self);
-PyObject* Signature_get_name(Signature *self);
-PyObject* Signature_get_email(Signature *self);
-PyObject* Signature_get_time(Signature *self);
-PyObject* Signature_get_offset(Signature *self);
-
 PyObject*
 build_signature(Object *obj, const git_signature *signature,
                 const char *encoding);
