@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -65,7 +65,7 @@ Tree_contains(Tree *self, PyObject *py_name)
 {
     if (Object__load((Object*)self) == NULL) { return -1; } // Lazy load
 
-    char *name = py_path_to_c_str(py_name);
+    char *name = pgit_encode_fsdefault(py_name);
     if (name == NULL)
         return -1;
 
@@ -119,6 +119,8 @@ Tree_iter(Tree *self)
 {
     TreeIter *iter;
 
+    if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
+
     iter = PyObject_New(TreeIter, &TreeIterType);
     if (iter) {
         Py_INCREF(self);
@@ -156,17 +158,14 @@ tree_getentry_by_index(const git_tree *tree, Repository *repo, PyObject *py_inde
 PyObject*
 tree_getentry_by_path(const git_tree *tree, Repository *repo, PyObject *py_path)
 {
-    char *path;
-    git_tree_entry *entry;
-    int err;
-
-    path = py_path_to_c_str(py_path);
+    char *path = pgit_encode_fsdefault(py_path);
     if (path == NULL) {
         PyErr_SetString(PyExc_TypeError, "Value must be a path string");
         return NULL;
     }
 
-    err = git_tree_entry_bypath(&entry, tree, path);
+    git_tree_entry *entry;
+    int err = git_tree_entry_bypath(&entry, tree, path);
     free(path);
 
     if (err == GIT_ENOTFOUND) {
@@ -203,7 +202,7 @@ Tree_divide(Tree *self, PyObject *value)
 
 
 PyDoc_STRVAR(Tree_diff_to_workdir__doc__,
-  "diff_to_workdir([flags, context_lines, interhunk_lines]) -> Diff\n"
+  "diff_to_workdir(flags: int = GIT_DIFF_NORMAL, context_lines: int = 3, interhunk_lines: int = 0) -> Diff\n"
   "\n"
   "Show the changes between the :py:class:`~pygit2.Tree` and the workdir.\n"
   "\n"
@@ -242,14 +241,14 @@ Tree_diff_to_workdir(Tree *self, PyObject *args)
 
 
 PyDoc_STRVAR(Tree_diff_to_index__doc__,
-  "diff_to_index(index, [flags, context_lines, interhunk_lines]) -> Diff\n"
+  "diff_to_index(index: Index, flags: int = GIT_DIFF_NORMAL, context_lines: int = 3, interhunk_lines: int = 0) -> Diff\n"
   "\n"
   "Show the changes between the index and a given :py:class:`~pygit2.Tree`.\n"
   "\n"
   "Parameters:\n"
   "\n"
-  "tree : :py:class:`~pygit2.Tree`\n"
-  "    The tree to diff.\n"
+  "index : :py:class:`~pygit2.Index`\n"
+  "    The index to diff.\n"
   "\n"
   "flag\n"
   "    A GIT_DIFF_* constant.\n"
@@ -270,7 +269,7 @@ Tree_diff_to_index(Tree *self, PyObject *args, PyObject *kwds)
     git_index *index;
     char *buffer;
     Py_ssize_t length;
-    PyObject *py_idx, *py_idx_ptr;
+    PyObject *py_idx;
     int err;
 
     if (!PyArg_ParseTuple(args, "O|IHH", &py_idx, &opts.flags,
@@ -278,43 +277,51 @@ Tree_diff_to_index(Tree *self, PyObject *args, PyObject *kwds)
                                         &opts.interhunk_lines))
         return NULL;
 
-    /*
-     * This is a hack to check whether we're passed an index, as I
-     * haven't found a good way to grab a type object for
-     * pygit2.index.Index.
-     */
-    if (!PyObject_GetAttrString(py_idx, "_index")) {
+    /* Check whether the first argument is an index.
+     * FIXME Uses duck typing. This would be easy and correct if we had
+     * _pygit2.Index. */
+    PyObject *pygit2_index = PyObject_GetAttrString(py_idx, "_index");
+    if (!pygit2_index) {
         PyErr_SetString(PyExc_TypeError, "argument must be an Index");
         return NULL;
     }
-    py_idx_ptr = PyObject_GetAttrString(py_idx, "_pointer");
+    Py_DECREF(pygit2_index);
+
+    /* Get git_index from cffi's pointer */
+    PyObject *py_idx_ptr = PyObject_GetAttrString(py_idx, "_pointer");
     if (!py_idx_ptr)
         return NULL;
 
     /* Here we need to do the opposite conversion from the _pointer getters */
     if (PyBytes_AsStringAndSize(py_idx_ptr, &buffer, &length))
-        return NULL;
+        goto error;
 
     if (length != sizeof(git_index *)) {
         PyErr_SetString(PyExc_TypeError, "passed value is not a pointer");
-        return NULL;
+        goto error;
     }
 
-    /* the "buffer" contains the pointer */
-    index = *((git_index **) buffer);
+    index = *((git_index **) buffer); /* the "buffer" contains the pointer */
 
+    /* Call git_diff_tree_to_index */
     if (Object__load((Object*)self) == NULL) { return NULL; } // Lazy load
 
     err = git_diff_tree_to_index(&diff, self->repo->repo, self->tree, index, &opts);
+    Py_DECREF(py_idx_ptr);
+
     if (err < 0)
         return Error_set(err);
 
     return wrap_diff(diff, self->repo);
+
+error:
+    Py_DECREF(py_idx_ptr);
+    return NULL;
 }
 
 
 PyDoc_STRVAR(Tree_diff_to_tree__doc__,
-  "diff_to_tree([tree, flags, context_lines, interhunk_lines, swap]) -> Diff\n"
+  "diff_to_tree([tree: Tree, flags: int = GIT_DIFF_NORMAL, context_lines: int = 3, interhunk_lines: int = 0, swap: bool = False]) -> Diff\n"
   "\n"
   "Show the changes between two trees.\n"
   "\n"
@@ -440,9 +447,8 @@ PyNumberMethods Tree_as_number = {
     0,                          /* nb_inplace_floor_divide */
     0,                          /* nb_inplace_true_divide */
     0,                          /* nb_index */
-// Python 3.5+
-//  0,                          /* nb_matrix_multiply */
-//  0,                          /* nb_inplace_matrix_multiply */
+    0,                          /* nb_matrix_multiply */
+    0,                          /* nb_inplace_matrix_multiply */
 };
 
 PyDoc_STRVAR(Tree__doc__, "Tree objects.");
@@ -457,7 +463,7 @@ PyTypeObject TreeType = {
     0,                                         /* tp_getattr        */
     0,                                         /* tp_setattr        */
     0,                                         /* tp_compare        */
-    0,                                         /* tp_repr           */
+    (reprfunc)Object_repr,                     /* tp_repr           */
     &Tree_as_number,                           /* tp_as_number      */
     &Tree_as_sequence,                         /* tp_as_sequence    */
     &Tree_as_mapping,                          /* tp_as_mapping     */
