@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -45,7 +45,7 @@ TreeBuilder_dealloc(TreeBuilder *self)
 
 
 PyDoc_STRVAR(TreeBuilder_insert__doc__,
-    "insert(name, oid, attr)\n"
+    "insert(name: str, oid: Oid, attr: int)\n"
     "\n"
     "Insert or replace an entry in the treebuilder.\n"
     "\n"
@@ -100,26 +100,23 @@ TreeBuilder_write(TreeBuilder *self)
 
 
 PyDoc_STRVAR(TreeBuilder_get__doc__,
-    "get(name) -> Object\n"
+    "get(name: str) -> Object\n"
     "\n"
     "Return the Object for the given name, or None if there is not.");
 
 PyObject *
 TreeBuilder_get(TreeBuilder *self, PyObject *py_filename)
 {
-    char *filename;
-    const git_tree_entry *entry_src;
-    git_tree_entry *entry;
-
-    filename = py_path_to_c_str(py_filename);
+    char *filename = pgit_encode_fsdefault(py_filename);
     if (filename == NULL)
         return NULL;
 
-    entry_src = git_treebuilder_get(self->bld, filename);
+    const git_tree_entry *entry_src = git_treebuilder_get(self->bld, filename);
     free(filename);
     if (entry_src == NULL)
         Py_RETURN_NONE;
 
+    git_tree_entry *entry;
     if (git_tree_entry_dup(&entry, entry_src) < 0) {
         PyErr_SetNone(PyExc_MemoryError);
         return NULL;
@@ -130,22 +127,20 @@ TreeBuilder_get(TreeBuilder *self, PyObject *py_filename)
 
 
 PyDoc_STRVAR(TreeBuilder_remove__doc__,
-    "remove(name)\n"
+    "remove(name: str)\n"
     "\n"
     "Remove an entry from the builder.");
 
 PyObject *
 TreeBuilder_remove(TreeBuilder *self, PyObject *py_filename)
 {
-    char *filename = py_path_to_c_str(py_filename);
-    int err = 0;
-
+    char *filename = pgit_encode_fsdefault(py_filename);
     if (filename == NULL)
         return NULL;
 
-    err = git_treebuilder_remove(self->bld, filename);
+    int err = git_treebuilder_remove(self->bld, filename);
     free(filename);
-    if (err < 0)
+    if (err)
         return Error_set(err);
 
     Py_RETURN_NONE;
