@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2019 The pygit2 contributors
+ * Copyright 2010-2021 The pygit2 contributors
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -39,6 +39,10 @@
 #  define PYGIT2_FN_UNUSED
 #endif
 
+#if defined(PYPY_VERSION)
+#define Py_FileSystemDefaultEncodeErrors "surrogateescape"
+#endif
+
 #define to_path(x) to_unicode(x, Py_FileSystemDefaultEncoding, "strict")
 #define to_encoding(x) PyUnicode_DecodeASCII(x, strlen(x), "strict")
 
@@ -57,8 +61,7 @@
 
 
 /* Utilities */
-#define to_unicode(x, encoding, errors)\
-        to_unicode_n(x, strlen(x), encoding, errors)
+#define to_unicode(x, encoding, errors) to_unicode_n(x, strlen(x), encoding, errors)
 
 PYGIT2_FN_UNUSED
 Py_LOCAL_INLINE(PyObject*)
@@ -81,24 +84,19 @@ to_unicode_n(const char *value, size_t len, const char *encoding,
     return PyUnicode_Decode(value, len, encoding, errors);
 }
 
-PYGIT2_FN_UNUSED
-Py_LOCAL_INLINE(PyObject*)
-to_bytes(const char * value)
-{
-    return PyBytes_FromString(value);
-}
+#define value_or_default(x, _default) ((x) == NULL ? (_default) : (x))
 
-char * py_str_to_c_str(PyObject *value, const char *encoding);
-const char *py_str_borrow_c_str(PyObject **tvaue, PyObject *value, const char *encoding);
+const char* pgit_borrow(PyObject *value);
+const char* pgit_borrow_encoding(PyObject *value, const char *encoding, const char *errors, PyObject **tvalue);
+char* pgit_encode(PyObject *value, const char *encoding);
+char* pgit_encode_fsdefault(PyObject *value);
 
-PyObject * get_pylist_from_git_strarray(git_strarray *strarray);
-int get_strarraygit_from_pylist(git_strarray *array, PyObject *pylist);
+
+//PyObject * get_pylist_from_git_strarray(git_strarray *strarray);
+//int get_strarraygit_from_pylist(git_strarray *array, PyObject *pylist);
 
 int py_object_to_otype(PyObject *py_type);
 
-#define py_path_to_c_str(py_path) \
-        py_str_to_c_str(py_path, Py_FileSystemDefaultEncoding)
-
 /* Helpers to make shorter PyMethodDef and PyGetSetDef blocks */
 #define METHOD(type, name, args)\
   {#name, (PyCFunction) type ## _ ## name, args, type ## _ ## name ## __doc__}
@@ -152,6 +150,15 @@ int py_object_to_otype(PyObject *py_type);
     if (PyModule_AddObject(module, #type, (PyObject*) & type ## Type) == -1)\
         return NULL;
 
+#define ADD_EXC(m, name, base)\
+    name = PyErr_NewException("_pygit2." #name, base, NULL);\
+    if (name == NULL) goto fail;\
+    Py_INCREF(name);\
+    if (PyModule_AddObject(m, #name, name)) {\
+        Py_DECREF(name);\
+        goto fail;\
+    }
+
 #define ADD_CONSTANT_INT(m, name) \
     if (PyModule_AddIntConstant(m, #name, name) == -1) return NULL;
 
