@@ -25,7 +25,7 @@
 
 """Tests for merging and information about it."""
 
-import os
+from pathlib import Path
 
 import pytest
 
@@ -82,7 +82,7 @@ def test_merge_invalid_hex(mergerepo):
 def test_merge_already_something_in_index(mergerepo):
     branch_head_hex = '03490f16b15a09913edb3a067a3dc67fbb8d41f1'
     branch_oid = mergerepo.get(branch_head_hex).id
-    with open(os.path.join(mergerepo.workdir, 'inindex.txt'), 'w') as f:
+    with (Path(mergerepo.workdir) / 'inindex.txt').open('w') as f:
         f.write('new content')
     mergerepo.index.add('inindex.txt')
     with pytest.raises(pygit2.GitError): mergerepo.merge(branch_oid)
@@ -253,4 +253,4 @@ def test_merge_octopus(mergerepo):
     assert index.conflicts is None
     merge_tree = index.write_tree()
 
-    assert merge_tree == merge_commits_tree
\ No newline at end of file
+    assert merge_tree == merge_commits_tree
