@@ -28,10 +28,7 @@
 # Standard Library
 import shutil
 import tempfile
-import os
-from os.path import join, realpath
 from pathlib import Path
-from urllib.request import pathname2url
 
 import pytest
 
@@ -50,18 +47,16 @@ def test_is_bare(testrepo):
 
 def test_get_path(testrepo_path):
     testrepo, path = testrepo_path
-    expected = realpath(join(path, '.git'))
-    assert realpath(testrepo.path) == expected
+    assert Path(testrepo.path).resolve() == (path / '.git').resolve()
 
 def test_get_workdir(testrepo_path):
     testrepo, path = testrepo_path
-    expected = realpath(path)
-    assert realpath(testrepo.workdir) == expected
+    assert Path(testrepo.workdir).resolve() == path.resolve()
 
 def test_set_workdir(testrepo):
     directory = tempfile.mkdtemp()
     testrepo.workdir = directory
-    assert realpath(testrepo.workdir) == realpath(directory)
+    assert Path(testrepo.workdir).resolve() == Path(directory).resolve()
 
 def test_checkout_ref(testrepo):
     ref_i18n = testrepo.lookup_reference('refs/heads/i18n')
@@ -82,6 +77,75 @@ def test_checkout_ref(testrepo):
     assert 'new' in head.tree
     assert 'bye.txt' not in testrepo.status()
 
+def test_checkout_callbacks(testrepo):
+    ref_i18n = testrepo.lookup_reference('refs/heads/i18n')
+
+    class MyCheckoutCallbacks(pygit2.CheckoutCallbacks):
+        def __init__(self):
+            super().__init__()
+            self.conflicting_paths = set()
+            self.updated_paths = set()
+
+        def checkout_notify_flags(self) -> int:
+            return pygit2.GIT_CHECKOUT_NOTIFY_CONFLICT | pygit2.GIT_CHECKOUT_NOTIFY_UPDATED
+
+        def checkout_notify(self, why, path, baseline, target, workdir):
+            if why == pygit2.GIT_CHECKOUT_NOTIFY_CONFLICT:
+                self.conflicting_paths.add(path)
+            elif why == pygit2.GIT_CHECKOUT_NOTIFY_UPDATED:
+                self.updated_paths.add(path)
+
+    # checkout i18n with conflicts and default strategy should not be possible
+    callbacks = MyCheckoutCallbacks()
+    with pytest.raises(pygit2.GitError): testrepo.checkout(ref_i18n, callbacks=callbacks)
+    # make sure the callbacks caught that
+    assert {'bye.txt'} == callbacks.conflicting_paths
+
+    # checkout i18n with GIT_CHECKOUT_FORCE
+    head = testrepo.head
+    head = testrepo[head.target]
+    assert 'new' not in head.tree
+    callbacks = MyCheckoutCallbacks()
+    testrepo.checkout(ref_i18n, strategy=pygit2.GIT_CHECKOUT_FORCE, callbacks=callbacks)
+    # make sure the callbacks caught the files affected by the checkout
+    assert set() == callbacks.conflicting_paths
+    assert {'bye.txt', 'new'} == callbacks.updated_paths
+
+def test_checkout_aborted_from_callbacks(testrepo):
+    ref_i18n = testrepo.lookup_reference('refs/heads/i18n')
+
+    def read_bye_txt():
+        return testrepo[testrepo.create_blob_fromworkdir("bye.txt")].data
+
+    s = testrepo.status()
+    assert s == {'bye.txt': pygit2.GIT_STATUS_WT_NEW}
+
+    class MyCheckoutCallbacks(pygit2.CheckoutCallbacks):
+        def __init__(self):
+            super().__init__()
+            self.invoked_times = 0
+
+        def checkout_notify(self, why, path, baseline, target, workdir):
+            self.invoked_times += 1
+            # skip one file so we're certain that NO files are affected,
+            # even if aborting the checkout from the second file
+            if self.invoked_times == 2:
+                raise InterruptedError("Stop the checkout!")
+
+    head = testrepo.head
+    head = testrepo[head.target]
+    assert 'new' not in head.tree
+    assert b'bye world\n' == read_bye_txt()
+    callbacks = MyCheckoutCallbacks()
+
+    # checkout i18n with GIT_CHECKOUT_FORCE - callbacks should prevent checkout from completing
+    with pytest.raises(InterruptedError):
+        testrepo.checkout(ref_i18n, strategy=pygit2.GIT_CHECKOUT_FORCE, callbacks=callbacks)
+
+    assert callbacks.invoked_times == 2
+    assert 'new' not in head.tree
+    assert b'bye world\n' == read_bye_txt()
+
 def test_checkout_branch(testrepo):
     branch_i18n = testrepo.lookup_branch('i18n')
 
@@ -103,7 +167,7 @@ def test_checkout_branch(testrepo):
 
 def test_checkout_index(testrepo):
     # some changes to working dir
-    with open(os.path.join(testrepo.workdir, 'hello.txt'), 'w') as f:
+    with (Path(testrepo.workdir) / 'hello.txt').open('w') as f:
         f.write('new content')
 
     # checkout index
@@ -113,7 +177,7 @@ def test_checkout_index(testrepo):
 
 def test_checkout_head(testrepo):
     # some changes to the index
-    with open(os.path.join(testrepo.workdir, 'bye.txt'), 'w') as f:
+    with (Path(testrepo.workdir) / 'bye.txt').open('w') as f:
         f.write('new content')
     testrepo.index.add('bye.txt')
 
@@ -128,11 +192,11 @@ def test_checkout_head(testrepo):
 
 def test_checkout_alternative_dir(testrepo):
     ref_i18n = testrepo.lookup_reference('refs/heads/i18n')
-    extra_dir = os.path.join(testrepo.workdir, 'extra-dir')
-    os.mkdir(extra_dir)
-    assert len(os.listdir(extra_dir)) == 0
+    extra_dir = Path(testrepo.workdir) / 'extra-dir'
+    extra_dir.mkdir()
+    assert len(list(extra_dir.iterdir())) == 0
     testrepo.checkout(ref_i18n, directory=extra_dir)
-    assert not len(os.listdir(extra_dir)) == 0
+    assert not len(list(extra_dir.iterdir())) == 0
 
 def test_checkout_paths(testrepo):
     ref_i18n = testrepo.lookup_reference('refs/heads/i18n')
@@ -189,7 +253,7 @@ def test_ahead_behind(testrepo):
 
 def test_reset_hard(testrepo):
     ref = "5ebeeebb320790caf276b9fc8b24546d63316533"
-    with open(os.path.join(testrepo.workdir, "hello.txt")) as f:
+    with (Path(testrepo.workdir) / "hello.txt").open() as f:
         lines = f.readlines()
     assert "hola mundo\n" in lines
     assert "bonjour le monde\n" in lines
@@ -199,7 +263,7 @@ def test_reset_hard(testrepo):
         pygit2.GIT_RESET_HARD)
     assert testrepo.head.target.hex == ref
 
-    with open(os.path.join(testrepo.workdir, "hello.txt")) as f:
+    with (Path(testrepo.workdir) / "hello.txt").open() as f:
         lines = f.readlines()
     #Hard reset will reset the working copy too
     assert "hola mundo\n" not in lines
@@ -207,7 +271,7 @@ def test_reset_hard(testrepo):
 
 def test_reset_soft(testrepo):
     ref = "5ebeeebb320790caf276b9fc8b24546d63316533"
-    with open(os.path.join(testrepo.workdir, "hello.txt")) as f:
+    with (Path(testrepo.workdir) / "hello.txt").open() as f:
         lines = f.readlines()
     assert "hola mundo\n" in lines
     assert "bonjour le monde\n" in lines
@@ -216,7 +280,7 @@ def test_reset_soft(testrepo):
         ref,
         pygit2.GIT_RESET_SOFT)
     assert testrepo.head.target.hex == ref
-    with open(os.path.join(testrepo.workdir, "hello.txt")) as f:
+    with (Path(testrepo.workdir) / "hello.txt").open() as f:
         lines = f.readlines()
     #Soft reset will not reset the working copy
     assert "hola mundo\n" in lines
@@ -228,7 +292,7 @@ def test_reset_soft(testrepo):
 
 def test_reset_mixed(testrepo):
     ref = "5ebeeebb320790caf276b9fc8b24546d63316533"
-    with open(os.path.join(testrepo.workdir, "hello.txt")) as f:
+    with (Path(testrepo.workdir) / "hello.txt").open() as f:
         lines = f.readlines()
     assert "hola mundo\n" in lines
     assert "bonjour le monde\n" in lines
@@ -239,7 +303,7 @@ def test_reset_mixed(testrepo):
 
     assert testrepo.head.target.hex == ref
 
-    with open(os.path.join(testrepo.workdir, "hello.txt")) as f:
+    with (Path(testrepo.workdir) / "hello.txt").open() as f:
         lines = f.readlines()
     #mixed reset will not reset the working copy
     assert "hola mundo\n" in lines
@@ -263,7 +327,7 @@ def test_stash(testrepo):
     assert [] == testrepo.listall_stashes()
 
     # some changes to working dir
-    with open(os.path.join(testrepo.workdir, 'hello.txt'), 'w') as f:
+    with (Path(testrepo.workdir) / 'hello.txt').open('w') as f:
         f.write('new content')
 
     testrepo.stash(sig, include_untracked=True, message=stash_message)
@@ -284,6 +348,107 @@ def test_stash(testrepo):
 
     with pytest.raises(KeyError): testrepo.stash_pop()
 
+def test_stash_progress_callback(testrepo):
+    sig = pygit2.Signature(name='Stasher', email='stasher@example.com', time=1641000000, offset=0)
+
+    # some changes to working dir
+    with (Path(testrepo.workdir) / 'hello.txt').open('w') as f:
+        f.write('new content')
+
+    # create the stash
+    testrepo.stash(sig, include_untracked=True, message="custom stash message")
+
+    progress_sequence = []
+
+    class MyStashApplyCallbacks(pygit2.StashApplyCallbacks):
+        def stash_apply_progress(self, progress: int):
+            progress_sequence.append(progress)
+
+    # apply the stash
+    testrepo.stash_apply(callbacks=MyStashApplyCallbacks())
+
+    # make sure the callbacks were notified of all the steps
+    assert progress_sequence == [
+        pygit2.GIT_STASH_APPLY_PROGRESS_LOADING_STASH,
+        pygit2.GIT_STASH_APPLY_PROGRESS_ANALYZE_INDEX,
+        pygit2.GIT_STASH_APPLY_PROGRESS_ANALYZE_MODIFIED,
+        pygit2.GIT_STASH_APPLY_PROGRESS_ANALYZE_UNTRACKED,
+        pygit2.GIT_STASH_APPLY_PROGRESS_CHECKOUT_UNTRACKED,
+        pygit2.GIT_STASH_APPLY_PROGRESS_CHECKOUT_MODIFIED,
+        pygit2.GIT_STASH_APPLY_PROGRESS_DONE,
+    ]
+
+def test_stash_aborted_from_callbacks(testrepo):
+    sig = pygit2.Signature(name='Stasher', email='stasher@example.com', time=1641000000, offset=0)
+
+    # some changes to working dir
+    with (Path(testrepo.workdir) / 'hello.txt').open('w') as f:
+        f.write('new content')
+    with (Path(testrepo.workdir) / 'untracked.txt').open('w') as f:
+        f.write('yo')
+
+    # create the stash
+    testrepo.stash(sig, include_untracked=True, message="custom stash message")
+
+    # define callbacks that will abort the unstash process
+    # just as libgit2 is ready to write the files to disk
+    class MyStashApplyCallbacks(pygit2.StashApplyCallbacks):
+        def stash_apply_progress(self, progress: int):
+            if progress == pygit2.GIT_STASH_APPLY_PROGRESS_CHECKOUT_UNTRACKED:
+                raise InterruptedError("Stop applying the stash!")
+
+    # attempt to apply and delete the stash; the callbacks will interrupt that
+    with pytest.raises(InterruptedError):
+        testrepo.stash_pop(callbacks=MyStashApplyCallbacks())
+
+    # we interrupted right before the checkout part of the unstashing process,
+    # so the untracked file shouldn't be here
+    assert not (Path(testrepo.workdir) / 'untracked.txt').exists()
+
+    # and hello.txt should be as it is on master
+    with (Path(testrepo.workdir) / 'hello.txt').open('r') as f:
+        assert f.read() == 'hello world\nhola mundo\nbonjour le monde\n'
+
+    # and since we didn't let stash_pop run to completion, the stash itself should still be here
+    repo_stashes = testrepo.listall_stashes()
+    assert 1 == len(repo_stashes)
+    assert repo_stashes[0].message == "On master: custom stash message"
+
+def test_stash_apply_checkout_options(testrepo):
+    sig = pygit2.Signature(name='Stasher', email='stasher@example.com', time=1641000000, offset=0)
+
+    hello_txt = Path(testrepo.workdir) / 'hello.txt'
+
+    # some changes to working dir
+    with hello_txt.open('w') as f:
+        f.write('stashed content')
+
+    # create the stash
+    testrepo.stash(sig, include_untracked=True, message="custom stash message")
+
+    # define callbacks that raise an InterruptedError when checkout detects a conflict
+    class MyStashApplyCallbacks(pygit2.StashApplyCallbacks):
+        def checkout_notify(self, why, path, baseline, target, workdir):
+            if why == pygit2.GIT_CHECKOUT_NOTIFY_CONFLICT:
+                raise InterruptedError("Applying the stash would create a conflict")
+
+    # overwrite hello.txt so that applying the stash would create a conflict
+    with hello_txt.open('w') as f:
+        f.write('conflicting content')
+
+    # apply the stash with the default (safe) strategy;
+    # the callbacks should detect a conflict on checkout
+    with pytest.raises(InterruptedError):
+        testrepo.stash_apply(strategy=pygit2.GIT_CHECKOUT_SAFE, callbacks=MyStashApplyCallbacks())
+
+    # hello.txt should be intact
+    with hello_txt.open('r') as f: assert f.read() == 'conflicting content'
+
+    # force apply the stash; this should work
+    testrepo.stash_apply(strategy=pygit2.GIT_CHECKOUT_FORCE, callbacks=MyStashApplyCallbacks())
+    with hello_txt.open('r') as f: assert f.read() == 'stashed content'
+
+
 def test_revert(testrepo):
     master = testrepo.head.peel()
     commit_to_revert = testrepo['4ec4389a8068641da2d6578db0419484972284c8']
@@ -316,7 +481,7 @@ def test_new_repo(tmp_path):
     oid = repo.write(pygit2.GIT_OBJ_BLOB, "Test")
     assert type(oid) == Oid
 
-    assert os.path.exists(os.path.join(tmp_path, '.git'))
+    assert (tmp_path / '.git').exists()
 
 
 def test_no_arg(tmp_path):
@@ -346,62 +511,45 @@ def test_keyword_arg_true(tmp_path):
 
 def test_discover_repo(tmp_path):
     repo = init_repository(tmp_path, False)
-    subdir = os.path.join(tmp_path, "test1", "test2")
-    os.makedirs(subdir)
-    assert repo.path == discover_repository(subdir)
+    subdir = tmp_path / "test1" / "test2"
+    subdir.mkdir(parents=True)
+    assert repo.path == discover_repository(str(subdir))
 
 @utils.fspath
 def test_discover_repo_aspath(tmp_path):
     repo = init_repository(Path(tmp_path), False)
     subdir = Path(tmp_path) / "test1" / "test2"
-    os.makedirs(subdir)
+    subdir.mkdir(parents=True)
     assert repo.path == discover_repository(subdir)
 
 def test_discover_repo_not_found():
     assert discover_repository(tempfile.tempdir) is None
 
 
-def test_bytes_string():
-    repo_path = b'./test/data/testrepo.git/'
-    pygit2.Repository(repo_path)
+def test_repository_init(barerepo_path):
+    barerepo, path = barerepo_path
+    assert isinstance(path, Path)
+    pygit2.Repository(path)
+    pygit2.Repository(str(path))
+    pygit2.Repository(bytes(path))
 
-def test_unicode_string():
-    # String is unicode because of unicode_literals
-    repo_path = './test/data/testrepo.git/'
-    pygit2.Repository(repo_path)
-
-def test_aspath():
-    repo_path = Path('./test/data/testrepo.git/')
-    pygit2.Repository(repo_path)
-
-
-def test_clone_repository(tmp_path):
-    repo_path = "./test/data/testrepo.git/"
-    repo = clone_repository(repo_path, tmp_path)
+def test_clone_repository(barerepo, tmp_path):
+    assert barerepo.is_bare
+    repo = clone_repository(Path(barerepo.path), tmp_path / 'clonepath')
+    assert not repo.is_empty
+    assert not repo.is_bare
+    repo = clone_repository(str(barerepo.path), str(tmp_path / 'clonestr'))
     assert not repo.is_empty
     assert not repo.is_bare
 
-def test_clone_repository_aspath(tmp_path):
-    repo_path = Path("./test/data/testrepo.git/")
-    repo = clone_repository(repo_path, Path(tmp_path))
-    assert not repo.is_empty
-    assert not repo.is_bare
-
-def test_clone_bare_repository(tmp_path):
-    repo_path = "./test/data/testrepo.git/"
-    repo = clone_repository(repo_path, tmp_path, bare=True)
+def test_clone_bare_repository(barerepo, tmp_path):
+    repo = clone_repository(barerepo.path, tmp_path / 'clone', bare=True)
     assert not repo.is_empty
     assert repo.is_bare
 
-def test_clone_repository_and_remote_callbacks(tmp_path):
-    src_repo_relpath = "./test/data/testrepo.git/"
-    repo_path = os.path.join(tmp_path, "clone-into")
-    url = pathname2url(os.path.realpath(src_repo_relpath))
-
-    if url.startswith('///'):
-        url = 'file:' + url
-    else:
-        url = 'file://' + url
+def test_clone_repository_and_remote_callbacks(barerepo, tmp_path):
+    url = Path(barerepo.path).resolve().as_uri()
+    repo_path = tmp_path / 'clone-into'
 
     def create_repository(path, bare):
         return init_repository(path, bare)
@@ -437,14 +585,11 @@ def test_clone_bad_credentials(tmp_path):
         clone_repository(url, tmp_path, callbacks=MyCallbacks())
     assert str(exc.value) == 'Unexpected error'
 
-def test_clone_with_checkout_branch(tmp_path):
+def test_clone_with_checkout_branch(barerepo, tmp_path):
     # create a test case which isolates the remote
-    test_repo = clone_repository('./test/data/testrepo.git',
-                                 os.path.join(tmp_path, 'testrepo-orig.git'),
-                                 bare=True)
+    test_repo = clone_repository(barerepo.path, tmp_path / 'testrepo-orig.git', bare=True)
     test_repo.create_branch('test', test_repo[test_repo.head.target])
-    repo = clone_repository(test_repo.path,
-                            os.path.join(tmp_path, 'testrepo.git'),
+    repo = clone_repository(test_repo.path, tmp_path / 'testrepo.git',
                             checkout_branch='test', bare=True)
     assert repo.lookup_reference('HEAD').target == 'refs/heads/test'
 
@@ -491,25 +636,25 @@ def test_clone_with_checkout_branch(tmp_path):
 
 def test_worktree(testrepo):
     worktree_name = 'foo'
-    worktree_dir = tempfile.mkdtemp()
+    worktree_dir = Path(tempfile.mkdtemp())
     # Delete temp path so that it's not present when we attempt to add the
     # worktree later
-    os.rmdir(worktree_dir)
+    worktree_dir.rmdir()
 
     def _check_worktree(worktree):
         # Confirm the name attribute matches the specified name
         assert worktree.name == worktree_name
         # Confirm the path attribute points to the correct path
-        assert os.path.realpath(worktree.path) == os.path.realpath(worktree_dir)
+        assert Path(worktree.path).resolve() == worktree_dir.resolve()
         # The "gitdir" in a worktree should be a file with a reference to
         # the actual gitdir. Let's make sure that the path exists and is a
         # file.
-        assert os.path.isfile(os.path.join(worktree_dir, '.git'))
+        assert (worktree_dir / '.git').is_file()
 
     # We should have zero worktrees
     assert testrepo.list_worktrees() == []
     # Add a worktree
-    worktree = testrepo.add_worktree(worktree_name, worktree_dir)
+    worktree = testrepo.add_worktree(worktree_name, str(worktree_dir))
     # Check that the worktree was added properly
     _check_worktree(worktree)
     # We should have one worktree now
@@ -536,13 +681,13 @@ def test_worktree_aspath(testrepo):
     worktree_dir = Path(tempfile.mkdtemp())
     # Delete temp path so that it's not present when we attempt to add the
     # worktree later
-    os.rmdir(worktree_dir)
+    worktree_dir.rmdir()
     testrepo.add_worktree(worktree_name, worktree_dir)
     assert testrepo.list_worktrees() == [worktree_name]
 
 def test_worktree_custom_ref(testrepo):
     worktree_name = 'foo'
-    worktree_dir = tempfile.mkdtemp()
+    worktree_dir = Path(tempfile.mkdtemp())
     branch_name = 'version1'
 
     # New branch based on head
@@ -550,10 +695,10 @@ def test_worktree_custom_ref(testrepo):
     worktree_ref = testrepo.branches.create(branch_name, tip)
     # Delete temp path so that it's not present when we attempt to add the
     # worktree later
-    os.rmdir(worktree_dir)
+    worktree_dir.rmdir()
 
     # Add a worktree for the given ref
-    worktree = testrepo.add_worktree(worktree_name, worktree_dir, worktree_ref)
+    worktree = testrepo.add_worktree(worktree_name, str(worktree_dir), worktree_ref)
     # We should have one worktree now
     assert testrepo.list_worktrees() == [worktree_name]
     # We should not have a branch of the same name
@@ -574,14 +719,14 @@ def test_worktree_custom_ref(testrepo):
     assert branch_name in testrepo.branches
 
 def test_open_extended(tmp_path):
-    with utils.TemporaryRepository('dirtyrepo.tar', tmp_path) as path:
+    with utils.TemporaryRepository('dirtyrepo.zip', tmp_path) as path:
         orig_repo = pygit2.Repository(path)
         assert not orig_repo.is_bare
         assert orig_repo.path
         assert orig_repo.workdir
 
         # GIT_REPOSITORY_OPEN_NO_SEARCH
-        subdir_path = os.path.join(path, "subdir")
+        subdir_path = path / "subdir"
         repo = pygit2.Repository(subdir_path)
         assert not repo.is_bare
         assert repo.path == orig_repo.path
@@ -591,7 +736,7 @@ def test_open_extended(tmp_path):
             repo = pygit2.Repository(subdir_path, pygit2.GIT_REPOSITORY_OPEN_NO_SEARCH)
 
         # GIT_REPOSITORY_OPEN_NO_DOTGIT
-        gitdir_path = join(path, ".git")
+        gitdir_path = path / '.git'
         with pytest.raises(pygit2.GitError):
             repo = pygit2.Repository(path, pygit2.GIT_REPOSITORY_OPEN_NO_DOTGIT)
 
@@ -610,7 +755,7 @@ def test_is_shallow(testrepo):
     assert not testrepo.is_shallow
 
     # create a dummy shallow file
-    with open(os.path.join(testrepo.path, 'shallow'), 'wt') as f:
+    with (Path(testrepo.path) / 'shallow').open('wt') as f:
         f.write('abcdef0123456789abcdef0123456789abcdef00\n')
 
     assert testrepo.is_shallow
